/**
 *  trafic
 *  Author: Patrick Taillandier, Benoit Gaudou & Philippe Caillou
 *  Description: step 6
 */

model traffic

global {
	float speed_parameter<-5.0;
	float proba_parameter<-0.05;
	file shapefile_buildings <- file("../includes/buildings.shp");
	file shapefile_roads <- file("../includes/roads.shp");
	geometry shape <- envelope(shapefile_roads);
	float step <- 10 #s;
	graph road_network;
	float cumulative_score<-0.0 update:cumulative_score+length(list(road) where (each.speed_rate<0.5));
	list<float> score_history<-[0.0] update:score_history+cumulative_score;
	
	init {
		create building from: shapefile_buildings;
		create road from: shapefile_roads;
		create inhabitant number: 1000{
			location <- any_location_in(one_of(building));
      	}
      	road_network <- as_edge_graph(road);
	}
	
	reflex update_speed {
		map<road,float> new_weights <- road as_map (each::each.shape.perimeter * each.speed_rate);
		road_network <- road_network with_weights new_weights;
	}
	
}

species inhabitant skills: [moving]{
	point target;
	float proba_leave <- proba_parameter; 
	float speed <- speed_parameter #km/#h;
	rgb color <- rnd_color(255);
	
	reflex leave when: (target = nil) and (flip(proba_leave)) {
		target <- any_location_in(one_of(building));
	}
	
	reflex move when: target != nil {
		do goto target: target on: road_network;
		if (location = target) {
			target <- nil;
		}	
	}
	
	aspect circle {
		draw circle(5) color: color;
	}
	
	aspect threeD{
		draw pyramid(5) color: color;
		draw sphere(2) at: location + {0,0,5} color: color;
	}
}

species building {
	int height <- 10 + rnd(20);
	aspect geom {
		draw shape color: #gray depth: height texture: ["../includes/roof_top.png","../includes/texture5.jpg"];
	}
}

species road {
	float capacity <- 1 + shape.perimeter/30;
	int nb_drivers <- 0 update: length(inhabitant at_distance 1);
	float speed_rate <- 1.0 update:  exp(-nb_drivers/capacity);
	string road_kind<- capacity<5 ? "small" : (capacity<10 ? "medium" : "large");
	aspect geom {
		draw (shape + 3 * speed_rate) color: #red;
	}
}
experiment traffic type: gui {
	output {
		display NbFree {
			chart "Number of cars and free roads"   
			type:series
			{
				data "free roads" value:(list(road) count (each.speed_rate=1));
				data "cars" value:length(list(inhabitant) where (each.target!=nil));
			}
			
		}
		display NiceNbFree {
			chart "Number of cars and free roads"   
			background: # darkblue 
			color: # lightgreen 
			axes: # lightgreen 
			title_font: font('Serif',32,'italic')
			tick_font: font('Monospaced',14, 'bold')
			label_font: font('Serif',18,'plain') 
			legend_font: font('SanSerif',18,'bold')
			x_range: 10 
			x_tick_unit: 3 
			x_serie_labels: ("T+" + cycle) 
			x_label: 'Step' 
			y_label: 'Cars/Free roads'
			type:series
			{
				data "free roads" value:length(list(road) where (each.speed_rate=1))  
					marker_shape: marker_empty  
					color: #red 
					style: spline 
					thickness:3;
				data "cars" value:length(list(inhabitant) where (each.target!=nil))  
					marker_shape: marker_circle 
					color: #yellow 
					thickness:1;
			}
			
		}
		display PropFreeHistData {
			chart "Congestion histogram"
			size: {0.5,1.0} position: {0, 0} 
			y_range:[0,350]
			type:histogram
			style: stack
			{
				data "free" value:length(list(road) where (each.speed_rate=1)) accumulate_values:true;			
				data "mostly free" value:length(list(road) where ((each.speed_rate>0.5 and each.speed_rate<1))) accumulate_values:true;			
				data "stuck" value:length(list(road) where (each.speed_rate<=0.5)) accumulate_values:true;			
			}		
			chart "Congestion histogram NC"
			size: {0.5,1.0} position: {0.5, 0} 
			type:histogram
			series_label_position:onchart
			y_range:[0,350]
			x_serie_labels:""
			{
				data "free" value:length(list(road) where (each.speed_rate=1)) accumulate_values:false;			
				data "mostly free" value:length(list(road) where ((each.speed_rate>0.5 and each.speed_rate<1))) accumulate_values:false;			
				data "stuck" value:length(list(road) where (each.speed_rate<=0.5)) accumulate_values:false;			
			}		
		}
		display PropFreeRadarSerie {
			chart "Congestion serie"
			size: {0.5,1.0} position: {0, 0} 
			type:series
			{
				datalist legend:["stuck","mostly free","free"] style:area
				value:[length(list(road) where (each.speed_rate<0.5)),
					length(list(road) where (each.speed_rate<1)),
					length(list(road) where (each.speed_rate<=1))
				] 
				color: [#red,#yellow,#green]
				accumulate_values:true;			
			}		
			chart "Congestion radar NC"
			size: {0.5,1.0} position: {0.5, 0} 
			type:radar
			x_serie_labels:["free","mostly free","stuck"]
			{
				data "status" value:[length(list(road) where (each.speed_rate=1)),length(list(road) where (each.speed_rate>0.5 and each.speed_rate<1)),length(list(road) where (each.speed_rate<0.5))];			
			}		
		}
		display DistribCapacity {
			chart "Roads by capacity"
			type:histogram			
			{
				datalist legend:(list(road sort_by each.capacity) frequency_of round(each.capacity)).keys 
					value:(list(road sort_by each.capacity) frequency_of round(each.capacity)).values;
			}		
		}
		display DistributionCeof {
			chart "Distribution of the speed coefs"   
			size: {1.0,0.5} position: {0, 0.5} 
			type:histogram			
			{
				datalist (distribution_of(list(road) collect each.speed_rate,5,0,1.25) at "legend") 
					value:(distribution_of(list(road) collect each.speed_rate,5,0,1.25) at "values");
			}
			chart "Distribution history of the speed coefs"   
			size: {1.0,0.5} position: {0, 0} 
			type:series
			{
				datalist (distribution_of(list(road) collect each.speed_rate,5,0,1.25) at "legend") 
					value:(distribution_of(list(road) collect each.speed_rate,5,0,1.25) at "values")
					marker:false;
			}
			
		}
		display map type: opengl{
			image "../includes/soil.jpg" refresh: false;
			species building aspect: geom refresh: false;
			species road aspect: geom ;
			species inhabitant aspect: threeD;
		}
	}
}

experiment 'trafic_explore_simple' type: batch repeat: 5 until: ( time > 100 ){
	permanent {
		display Comparison_hist {
			chart "Congestion score evolution" type: series {
				datalist legend:(simulations collect each.name) 
					value: (simulations collect each.score_history )
					accumulate_values:false style:dot;
			}
		}	
	}
}

experiment 'trafic_explore_1D' type: batch repeat: 5 until: ( time > 20 ){
	parameter 'Proba parameter' var: proba_parameter among: [ 0.05 , 0.1 , 0.15 ];
	method exploration ;
	permanent {
		display Comparison {
			chart "Congestion score" type: series x_label:"Probability" x_serie_labels:[ 0.05 , 0.1 , 0.15 ]{
				data "score" value: mean(simulations collect each.cumulative_score ) 
				y_err_values:[min(simulations collect each.cumulative_score ),max(simulations collect each.cumulative_score )] 
				thickness:3 color: #blue ;
			}
		}	
	}
}

experiment 'trafic_explore2D' type: batch repeat: 5 until: ( time > 20 ){
	method exploration ;
	list<float> spp<-[ 0.5 , 5.0 , 50.0 ];
	list<float> prp<-[ 0.05 , 0.1 , 0.15 ];
	parameter 'Speed parameter' var: speed_parameter among: [ 0.5 , 5.0 , 50.0 ];
	parameter 'Proba parameter' var: proba_parameter among: [ 0.05 , 0.1 , 0.15 ];
	list<list<float>> scoreexp<-[[0,0,0],[0,0,0],[0,0,0]];

	int startt<- rnd(10000000);
	
	init {
		save ["speed_parameter","proba_parameter", "average","min","max"] to: "../results/exp"+startt+".csv" rewrite:true format:"csv";
	}

	reflex end_of_runs
	{
		scoreexp[spp index_of speed_parameter][prp index_of proba_parameter]<-mean(simulations collect each.cumulative_score );
		write("result: x "+spp index_of speed_parameter+" y "+prp index_of proba_parameter+" v "+mean(simulations collect each.cumulative_score));
		write(scoreexp);
		save [speed_parameter,proba_parameter, mean(simulations collect each.cumulative_score ),min(simulations collect each.cumulative_score ),max(simulations collect each.cumulative_score )] to: "../results/exp"+startt+".csv" rewrite:false format:"csv";
	}
	permanent {
		display Comparison {
			chart "Congestion score" type: heatmap 
			x_serie_labels:spp
			y_serie_labels:prp			
			{
				data  "score"
				color: [#green,#yellow,#red]
				value: scoreexp;						
			}		
		}	
	}
}

experiment 'trafic_optimize' type: batch repeat: 3 until: ( time > 20 ){
	parameter 'Speed parameter' var: speed_parameter min:1.0 max:50.0 step:1.0;
	parameter 'Proba parameter' var: proba_parameter min:0.05 max:0.5 step:0.05;
	method hill_climbing iter_max: 30 minimize: cumulative_score aggregation: "max";
	permanent {
		display Comparison {
			chart "Score" type: series 
			size: {1.0,0.5} position: {0.0, 0} 
			{
				data "Score" 
				value: mean(simulations collect each.cumulative_score ) 
				y_err_values:[min(simulations collect each.cumulative_score ),max(simulations collect each.cumulative_score )] 
				color: #blue 
				marker:false
				thickness:3;
			}
			
			chart "parameters" type: xy 
			size: {1.0,0.5} position: {0.0, 0.5} 
			{
				data "speed/proba" 
				value: [speed_parameter,proba_parameter]
				marker:false 
				thickness:2
				accumulate_values:true color: #blue ;
				data "last speed/proba" 
				value: [speed_parameter,proba_parameter]
				marker_shape:marker_circle 
				marker_size:3
				accumulate_values:false color: #red ;
			}
		}	
	}

}

